/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.commands.adds;


import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.gef.commands.Command;

import vocleditor.editors.VOCLEditorConstants;
import vocleditor.figure.ClassifierRoleFigure;
import vocleditor.model.AttributeElement;
import vocleditor.model.VOCLModelManager;
import vocleditor.model.VarDecElement;
import VOCLModel.Attribute;
import VOCLModel.Constraint;
import VOCLModel.Method;
import VOCLModel.NavExpContainer;
import VOCLModel.ClassifierRole;
import VOCLModel.VOCLExpression;
import VOCLModel.VOCLModel4Factory;
import VOCLModel.VariableDeclaration;



public class ClassifierPropsCommand extends Command {
	
	// Names for Label and Description of this Command
	private static final String ConnectionCommand_Label 		= "Add Properties";
	private static final String ConnectionCommand_Description 	= "Adds an property to a classifier";
	
	// the associated ClassifierRole & ClassifierRoleFigure
	// and the Container-Class
	protected ClassifierRole source;
	protected ClassifierRoleFigure classFigure;
	protected NavExpContainer container;
	protected Constraint constraint;
	private VOCLModel4Factory mfc = VOCLModelManager.getFactory();
	

	
	private Hashtable values=new Hashtable();
	private Hashtable oldvalues=new Hashtable();
	

	
	/**
	 * Constructor, simply calls the superclass
	 * with the ConnectionLabel as parameter.
	 */
	public ClassifierPropsCommand() {
		super(ConnectionCommand_Label);
	}
	

	/**
	 * This method returns true, if a source (ClassifierRole, see class-members)
	 * is specified. 
	 */
	public boolean canExecute() {
		if (source == null)return false;
		else return true;
	}
	
	/**
	 * This method handles the execution of adding an Attribute or a Method
	 * to a class (ClassifierRole). Therefor it uses a NavExpModelFactory
	 * to create an Attribute or Method and sets the the corresponding
	 * references to the ClassifierRole.
	 * Finally the ClassifierRoleFigure is updated.
	 */
	public void execute() {
		initValues(oldvalues);
		
		String globalName=(String)values.get(VOCLEditorConstants.CLASS_VAR_NAME);
		
		if(globalName.trim().length()>0){
			if(source.getCallVar()!=null)
				source.getCallVar().setVarName(globalName);
			else{
					VOCLModel4Factory mfc = VOCLModelManager.getFactory();
					VariableDeclaration callVar = mfc.createVariableDeclaration(); 
					callVar.setVarName(globalName);
					callVar.setType(source.getType());
					constraint.getVariableDeclaration().add(callVar);
					source.setCallVar(callVar);
			}
		}else{
			if(source.getCallVar()!=null){
				constraint.getVariableDeclaration().remove(source.getCallVar());
					source.setCallVar(null);
			}
			
		}
		
		
		insertAttributes((List) values.get(VOCLEditorConstants.ATTRIBUTES));	
		setMethod(values);
	}
	
	private void setMethod(Hashtable table){
		if(source.getMethod()!=null){
			Method met=source.getMethod();
			constraint.getVariableDeclaration().removeAll(met.getCall());
			constraint.getVariableDeclaration().remove(met.getRReturn());
			source.setMethod(null);
		}
		if(table.get(VOCLEditorConstants.METHOD_NAME)!=null){
			Method met=mfc.createMethod();
			met.setType((String) table.get(VOCLEditorConstants.METHOD_TYPE));
			met.setName((String) table.get(VOCLEditorConstants.METHOD_NAME));
			if(table.get(VOCLEditorConstants.METHOD_RETURN_NAME)!=null){
				VariableDeclaration variable=mfc.createVariableDeclaration();
				variable.setVarName((String) table.get(VOCLEditorConstants.METHOD_RETURN_NAME));
				variable.setType((String) table.get(VOCLEditorConstants.METHOD_TYPE));
				constraint.getVariableDeclaration().add(variable);
				met.setRReturn(variable);				
			}
			
			List parlist=(List) table.get(VOCLEditorConstants.METHOD_PARAMETER);
			Iterator it=parlist.iterator();
			while(it.hasNext()){
				VarDecElement varElem=(VarDecElement) it.next();
				VariableDeclaration var=mfc.createVariableDeclaration();
				var.setType(varElem.getType());
				var.setVarName(varElem.getVarname());
				met.getCall().add(var);
				constraint.getVariableDeclaration().add(var);
			}
			source.setMethod(met);
		}			
	}

	private void insertAttributes(List list){
		Iterator i=source.getAttribute().iterator();
		while(i.hasNext()){
			Attribute att=(Attribute)i.next();
			if(att.getVariableDeclaration()!=null)
				constraint.getVariableDeclaration().remove(att.getVariableDeclaration());
		}
		source.getAttribute().clear();

		Iterator it=list.iterator();
		while(it.hasNext()){
			AttributeElement attElem=(AttributeElement)it.next();
			Attribute att=mfc.createAttribute();
			att.setName(attElem.getAttName());
			att.setType(attElem.getType());
			if(attElem.isVar()){
				VariableDeclaration var=mfc.createVariableDeclaration();
				var.setVarName(attElem.getVarname());
				var.setType(attElem.getType());
				att.setValue("");
				att.setVariableDeclaration(var);
				constraint.getVariableDeclaration().add(var);
			}else{
				att.setValue(attElem.getValue());
			}
			source.getAttribute().add(att);	
		}			
	}


	/**
	 * Retrieves the description.
	 */
	public String getLabel() {
		return ConnectionCommand_Label;
	}

	/**
	 * Retrieves the corresponding ClassifierRole
	 * @return - the ClassifierRole
	 */
	public ClassifierRole getSource() {
		return source;
	}

	/**
	 * Redo this action means 
	 * simply execute it (again).
	 */
	public void redo() {
		String globalName=(String)values.get(VOCLEditorConstants.CLASS_VAR_NAME);
		
		if(globalName.trim().length()>0){
			if(source.getCallVar()!=null)
				source.getCallVar().setVarName(globalName);
			else{
					VOCLModel4Factory mfc = VOCLModelManager.getFactory();
					VariableDeclaration callVar = mfc.createVariableDeclaration(); 
					callVar.setVarName(globalName);
					callVar.setType(source.getType());
					constraint.getVariableDeclaration().add(callVar);
					source.setCallVar(callVar);
			}
		}else{
			if(source.getCallVar()!=null){
				constraint.getVariableDeclaration().remove(source.getCallVar());
					source.setCallVar(null);
			}
			
		}
		insertAttributes((List) values.get(VOCLEditorConstants.ATTRIBUTES));
		setMethod(values);
	}

	/**
	 * Sets the ClassifierRole for this Action.
	 * @param newSource - the ClassifierRole
	 */
	public void setSource(ClassifierRole newSource) {
		source = newSource;
	}

	/**
	 * Implemented as:
	 * 1. remove last Attribute from list
	 * 2. update figure
	 */
	public void undo() {	
		String globalName=(String)oldvalues.get(VOCLEditorConstants.CLASS_VAR_NAME);	
		if(globalName.trim().length()>0){
			if(source.getCallVar()!=null)
				source.getCallVar().setVarName(globalName);
			else{
					VOCLModel4Factory mfc = VOCLModelManager.getFactory();
					VariableDeclaration callVar = mfc.createVariableDeclaration(); 
					callVar.setVarName(globalName);
					callVar.setType(source.getType());
					constraint.getVariableDeclaration().add(callVar);
					source.setCallVar(callVar);
			}
		}else{
			if(source.getCallVar()!=null){
				constraint.getVariableDeclaration().remove(source.getCallVar());
					source.setCallVar(null);
			}
			
		}
		insertAttributes((List) oldvalues.get(VOCLEditorConstants.ATTRIBUTES));	
		setMethod(oldvalues);
	}
	
	/**
	 * Returns the container-class of this model.
	 * @return - a NavExpContainer
	 */
	public NavExpContainer getContainer() {
		return container;
	}

	/**
	 * Sets the container-class for this model.
	 * @param container
	 */
	public void setContainer(NavExpContainer container) {
		this.container = container;
		constraint=null;
		VOCLExpression exp=container;
		while(constraint==null){
			if(exp.eContainer() instanceof Constraint)constraint=exp.getConstraint();
			else exp=(VOCLExpression) exp.eContainer();	
		}
	}
	
	
	
	private void initValues(Hashtable table){
		List attlist=new LinkedList();
		List parlist=new LinkedList();
		initList(attlist);
		
		String varname="";
		if(source.getCallVar()!=null)varname=source.getCallVar().getVarName();
		table.put(VOCLEditorConstants.CLASS_VAR_NAME, varname);
		table.put(VOCLEditorConstants.CLASS_NAME,source.getType());
		table.put(VOCLEditorConstants.ATTRIBUTES,attlist);
		if(source.getMethod()!=null){
			Method met=source.getMethod();
			table.put(VOCLEditorConstants.METHOD_NAME,met.getName());
			table.put(VOCLEditorConstants.METHOD_TYPE,met.getType());
			if(met.getRReturn()!=null){
				table.put(VOCLEditorConstants.METHOD_RETURN_NAME,met.getRReturn().getVarName());
			}
			Iterator it=met.getCall().iterator();
			while(it.hasNext()){
				VariableDeclaration var=(VariableDeclaration) it.next();
				VarDecElement varElem=new VarDecElement(var.getType(),var.getVarName());
				parlist.add(varElem);
			}
		}
		table.put(VOCLEditorConstants.METHOD_PARAMETER,parlist);
	}
	

	private void initList(List list){
		list.clear();
		Iterator it=source.getAttribute().iterator();
		while(it.hasNext()){
			Attribute att=(Attribute)it.next();
			AttributeElement attElem=new AttributeElement();
			attElem.setAttName(att.getName());
			attElem.setType(att.getType());
			if(att.getVariableDeclaration()!=null){
				VariableDeclaration var=att.getVariableDeclaration();
				attElem.setVarname(var.getVarName());
				attElem.setValue("");
				attElem.setVar(true);
			}else{
				attElem.setValue(att.getValue());
				attElem.setVar(false);
			}
			list.add(attElem);	
		}				
	}


	/**
	 * @return
	 */
	public Hashtable getValues() {
		initValues(values);
		return values;
	}

	/**
	 * @param hashtable
	 */
	public void setValues(Hashtable hashtable) {
		values.clear();
		values.putAll(hashtable);
	}

}