/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.commands.connect;

import vocleditor.model.VOCLModelManager;
import VOCLModel.ClassifierRole;
import VOCLModel.Constraint;
import VOCLModel.SetClassifierRole;
import VOCLModel.VOCLExpression;
import VOCLModel.VOCLModel4Factory;
import VOCLModel.VariableDeclaration;



/**
 * Class       :ConnectCommand<br/>
 * Package     : vocleditor.commands.connect<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : Command to connect two ClassifierRoles with AssociationRole
 * <br/>
 * @author vilapower
 */
public class IsInDisconnectCommand extends NavDisconnectCommand {
	
	private VariableDeclaration var;
	private Constraint constraint;
	protected VOCLModel4Factory mfc = VOCLModelManager.getFactory();
	
	/**
	 * @see org.eclipse.gef.commands.Command
	 */			
	public IsInDisconnectCommand() {
		super();
	}
	
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		constraint=null;
		VOCLExpression exp=(VOCLExpression)container;
		while(constraint==null){
			if(exp.eContainer() instanceof Constraint)
				constraint=(Constraint) exp.getConstraint();
			else exp=(VOCLExpression) exp.eContainer();	
		}	
		var=mfc.createVariableDeclaration();
		if(target instanceof ClassifierRole){
			var.setVarName(target.getCallVar().getVarName());
			var.setType(target.getCallVar().getType());
		}else{
			var.setVarName(source.getCallVar().getVarName());
			var.setType(source.getCallVar().getType());
		}
		super.execute();
	}
	
	//delete connection
	protected void deleteConnection(){
		constraint.getVariableDeclaration().add(var);
		if(target instanceof ClassifierRole)
			target.setCallVar(var);
		else
			source.setCallVar(var);
		
		super.deleteConnection();
	}
	
	/**
	* @see org.eclipse.gef.commands.Command#redo()
	*/
	public void redo() {
		super.redo();			
	}

	/**
	* @see org.eclipse.gef.commands.Command#undo()
	*/	
	public void undo() {
		constraint.getVariableDeclaration().remove(var);
		if(target instanceof ClassifierRole)
			target.setCallVar(((SetClassifierRole)source).getPredicateContainer().getIterator());
		else
			source.setCallVar(((SetClassifierRole)target).getPredicateContainer().getIterator());
		
		super.undo();
	}
	
}