/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.commands.connect;

import org.eclipse.gef.commands.Command;

import VOCLModel.AssociationEndRoleSource;
import VOCLModel.AssociationEndRoleTarget;
import VOCLModel.AssociationRole;
import VOCLModel.ClassifierNode;
import VOCLModel.NavExpContainer;


/**
 * Class       :ConnectCommand<br/>
 * Package     : vocleditor.commands.connect<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : Command to connect two ClassifierRoles with AssociationRole
 * <br/>
 * @author vilapower
 */
public class NavDisconnectCommand extends Command {
	


	protected AssociationRole association;
	protected NavExpContainer container;

	protected ClassifierNode source=null;
	protected ClassifierNode target=null;
	protected AssociationEndRoleSource assERS=null ;
	protected AssociationEndRoleTarget assERT=null ;	
	
	/**
	 * @see org.eclipse.gef.commands.Command
	 */			
	public NavDisconnectCommand() {
		super();
	}
	
	/**
	* @see org.eclipse.gef.commands.Command#canExecute()
	*/	
	public boolean canExecute() {
		if(source==null || target==null || association==null) return false;
		return true;
	}
	
	/**
	* @see org.eclipse.gef.commands.Command#execute()
	*/
	public void execute() {
		deleteConnection();
	}
	

	//delete connection
	protected void deleteConnection(){
		target.getTarget().remove(assERT);
		source.getSource().remove(assERS);
		assERS.setType(null);
		assERT.setType(null);
		association.setSource(null);
		association.setTarget(null);
		container.getAssociationRole().remove(association);
	}
	


	/**
	 * returns association
	 * @return association the <code>AssociationRole</code>
	 */
	public AssociationRole getAssociation(){
		return association;
	}
	

	/**
	* @see org.eclipse.gef.commands.Command#redo()
	*/
	public void redo() {
		//redo delete connection
		deleteConnection();			
	}


	/**
	* set association
	* @param newArc the <code>AssociationRole</code>
	*/
	public void setAssociation(AssociationRole newArc){
		association = newArc;
	}
	
	/**
	* @see org.eclipse.gef.commands.Command#undo()
	*/	
	public void undo() {
		//create deleted connection
		container.getAssociationRole().add(association);		
		association.setSource(assERS);
		assERS.setAssociationRole(association);
		association.setTarget(assERT);
		assERT.setAssociationRole(association);		
		source.getSource().add(assERS);
		target.getTarget().add(assERT);
		assERS.setType(source);	
		assERT.setType(target);	
	}
	
	/**
	 * returns container that contains source ,target node etc. 
	 * @return container the <code>NavExpContainer</code>
	 */
	public NavExpContainer getContainer() {
		return container;
	}

	/**
	 * set container that contains source ,target node etc. 
	 * @param container <code>NavExpContainer</code>
	 */
	public void setContainer(NavExpContainer container) {
		this.container = container;
	}


	/**
	 * @return
	 */
	public AssociationEndRoleSource getAssERS() {
		return assERS;
	}

	/**
	 * @return
	 */
	public AssociationEndRoleTarget getAssERT() {
		return assERT;
	}

	/**
	 * @return
	 */
	public ClassifierNode getSource() {
		return source;
	}

	/**
	 * @return
	 */
	public ClassifierNode getTarget() {
		return target;
	}

	/**
	 * @param source
	 */
	public void setAssERS(AssociationEndRoleSource source) {
		assERS = source;
	}

	/**
	 * @param target
	 */
	public void setAssERT(AssociationEndRoleTarget target) {
		assERT = target;
	}

	/**
	 * @param role
	 */
	public void setSource(ClassifierNode role) {
		source = role;
	}

	/**
	 * @param role
	 */
	public void setTarget(ClassifierNode role) {
		target = role;
	}



}