/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.commands.create;

import vocleditor.model.VOCLModelManager;
import VOCLModel.BoolExp;
import VOCLModel.BoolType;
import VOCLModel.ExpressionType;
import VOCLModel.NavExpContainer;
import VOCLModel.VOCLModel4Factory;
import VOCLModel.VOCLSash;

/**
 * @author eddi
 */
public class CreateBoolExpCommand extends CreateExpressionCommand {
	

	protected VOCLSash sash;
	private int kind=0;
	private boolean vertical=true;

	/**
	 * Execution of this command creates a new BoolExpression
	 * and adds it to the expression container 
	 */
	public void execute() {
		VOCLModel4Factory mfc = VOCLModelManager.getFactory();
		sash=mfc.createVOCLSash();
		sash.setVertikal(isVertical());
		((BoolExp)expression).setVOCLSash(sash);
		((BoolExp)expression).setProportion(0.5);
		switch(getKind()){
			case 0:
				((BoolExp)expression).setKind(BoolType.get(0));
				break;
			case 1:
				((BoolExp)expression).setKind(BoolType.get(1));
				break;
			case 2:	
				((BoolExp)expression).setKind(BoolType.get(2));	
				break;
			case 3:
				((BoolExp)expression).setKind(BoolType.get(3));
				break;
		}
		
		NavExpContainer container1=mfc.createNavExpContainer();
		((BoolExp)expression).setOp1(container1);
		dublicate.dublicateContainer(container1,navexp);
		container1.setExptype(ExpressionType.OP_1_LITERAL);
		
		NavExpContainer container2=mfc.createNavExpContainer();
		((BoolExp)expression).setOp2(container2);
		dublicate.dublicateContainer(container2,navexp);
		container2.setExptype(ExpressionType.OP_2_LITERAL);

		super.execute();
	}

	/**
	 * Getter for the kind of this method
	 * @return - an int-value of the kind (and, or, etc)
	 */
	public int getKind() {
		return kind;
	}

	/**
	 * Checks if the created BoolExpression should be layouted
	 * vertically or not
	 * @return - true if the layout should be vertically
	 */
	public boolean isVertical() {
		return vertical;
	}

	/**
	 * Sets the Kind of this BoolExpression
	 * @param i - the interegr-code of the kind
	 */
	public void setKind(int i) {
		kind = i;
	}

	/**
	 * Sets the vertical-flag for the layout to the given
	 * value
	 * @param b - the new value for the layout-flag (true means vertical)
	 */
	public void setVertical(boolean b) {
		vertical = b;
	}

}
