/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.commands.create;




import java.util.LinkedList;
import java.util.List;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;

import vocleditor.model.VOCLModelManager;
import VOCLModel.Constraint;
import VOCLModel.Context;
import VOCLModel.ExpressionType;
import VOCLModel.Kind;
import VOCLModel.NavExpContainer;
import VOCLModel.VOCLDiagramm;
import VOCLModel.VOCLModel4Factory;
import VOCLModel.VariableDeclaration;


public class CreateConstraintCommand
	extends org.eclipse.gef.commands.Command
{
	private static final String	CreateCommand_Label = "CreateConstraintCommand";
	private Constraint constraint;
	private VariableDeclaration varDec;
	private VOCLDiagramm parent;
	private Context context;
	private NavExpContainer container;
	private Point loc;
	private Dimension dim;


	private List parameter=new LinkedList();
	private boolean classifier;
	private String nameOfContext;
	private int contextKind;
	private String variableName;
	private String typeOfContext;
	private String nameOfConstraint;
	

	/**
	 * The constructor of this command.
	 */
	public CreateConstraintCommand() {
		super();
	}

	/**
	 * Execution of this class creates a new constraint and 
	 * adds it to the container
	 */
	public void execute() {
		initilize();
		if (loc != null) {
			constraint.setX(loc.x);
			constraint.setY(loc.y);
			constraint.setWidth(320);
			constraint.setHeight(200);
		}
	}
	
	/**
	 * Initializes the constraint (e.g. the kind)
	 * and creates the needed variable-declarations.
	 */
	private void initilize(){
		VOCLModel4Factory mfc = VOCLModelManager.getFactory();
		switch(getContextKind()){
			case 0:
				constraint.setKind(Kind.get(0));
				break;
			case 1:
				constraint.setKind(Kind.get(1));
				break;
			case 2:	
				constraint.setKind(Kind.get(2));	
		}
		context= mfc.createContext();
		context.setName(getNameOfContext());
		context.setIsClassifier(isClassifier());
		if(isClassifier()){
			varDec=mfc.createVariableDeclaration();
			varDec.setVarName(getVariableName());
			varDec.setType(getNameOfContext());
			context.getVariableDeclaration().add(varDec);
			context.setType("");
			constraint.getVariableDeclaration().add(varDec);		
		}
		else{			
			for(int i=0 ; i<parameter.size();i++){
				varDec=mfc.createVariableDeclaration();
				List list=(List)parameter.get(i);
				varDec.setVarName((String)list.get(0));
				varDec.setType((String)list.get(1));
				constraint.getVariableDeclaration().add(varDec);
				context.getVariableDeclaration().add(varDec);
			}
			context.setType(getTypeOfContext());
		}
		constraint.setContext(context);
		container=mfc.createNavExpContainer();
		container.setExptype(ExpressionType.EXP_LITERAL);
		constraint.setVOCLExpression(container);
		constraint.setName(getNameOfConstraint());
		parent.getConstraint().add(constraint);
	}
	
	
	
	/**
	 * Redos an undo previously made.
	 */
	public void redo() {
		parent.getConstraint().add(constraint); 
	}
	
	/**
	 * Setter for the constraint
	 * @param constraint
	 */
	public void setConstraint( Constraint constraint ) {
		this.constraint = constraint;
	}
	
	/**
	 * Setter for the location
	 * @param loc - a point holding the new location
	 */
	public void setLocation (Point loc) {
		this.loc = loc;
	}
	
	/**
	 * Undos the action
	 */
	public void undo() {	
		parent.getConstraint().remove(constraint);
	}
	

	/**
	 * Retrieves the dimension of the constraint
	 * @return - the dimension
	 */
	public Dimension getDim() {
		return dim;
	}

	/**
	 * Sets the new dimension of the constraint
	 * @param dimension
	 */
	public void setDim(Dimension dimension) {
		dim = dimension;
	}

	/**
	 * Setter for the parameters in a list
	 * @param list holding the parameters
	 */
	public void setParameter(List list) {
		parameter = list;
	}

	/**
	 * Checks if this is a classifier
	 * @return true, if it is a classifier
	 */
	public boolean isClassifier() {
		return classifier;
	}
	
	/**
	 * Sets classifier
	 * @param b - the new value for the classifier-indicator
	 */
	public void setClassifier(boolean b) {
		classifier = b;
	}

	/**
	 * Retrieves the name of the context
	 * @return - the string-value of the contexts name
	 */
	public String getNameOfContext() {
		return nameOfContext;
	}

	/**
	 * Setter for the name of the context
	 * @param string - the new name of the context
	 */
	public void setNameOfContext(String string) {
		nameOfContext = string;
	}

	/**
	 * Getter for the context kind
	 * @return - the integer-code of the contexts kind
	 */
	public int getContextKind() {
		return contextKind;
	}

	/**
	 * Setter for the context kind
	 * @param i - the new value for the kind as integer.
	 */
	public void setContextKind(int i) {
		contextKind = i;
	}

	/**
	 * Retrieves the variables name of the context
	 * @return - the string value of the variables name.
	 */
	public String getVariableName() {
		return variableName;
	}

	/**
	 * Setter for the contexts variable name
	 * @param string - the new name
	 */
	public void setVariableName(String string) {
		variableName = string;
	}

	/**
	 * Getter for the type of the context as string value
	 * @return - the type of the context
	 */
	public String getTypeOfContext() {
		return typeOfContext;
	}

	/**
	 * Setter for the type of the context
	 * @param string - the new value of the type as string
	 */
	public void setTypeOfContext(String string) {
		typeOfContext = string;
	}

	/**
	 * Getter for the name of the constraint
	 * @return - the string value of the constraints name
	 */
	public String getNameOfConstraint() {
		return nameOfConstraint;
	}

	/**
	 * Setter for the constraints name
	 * @param string - the new name
	 */
	public void setNameOfConstraint(String string) {
		nameOfConstraint = string;
	}

	/**
	 * Getter for the parent, where the constraint is nested 
	 * inside
	 * @return - the VOCLDiagram parent
	 */
	public VOCLDiagramm getParent() {
		return parent;
	}

	/**
	 * Setter for the parent of the constraint
	 * @param diagramm
	 */
	public void setParent(VOCLDiagramm diagramm) {
		parent = diagramm;
	}

}
