/**
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.commands.create;

import org.eclipse.gef.commands.Command;

import vocleditor.model.DublicateExpression;
import VOCLModel.BoolExp;
import VOCLModel.Constraint;
import VOCLModel.ExpressionType;
import VOCLModel.IfExp;
import VOCLModel.LetExp;
import VOCLModel.NavExpContainer;
import VOCLModel.PredicateContainer;
import VOCLModel.VOCLExpression;

/**
 * @author eddi
 */
public abstract class CreateExpressionCommand extends Command {
	
	//container where the new expression will be insert
	protected VOCLExpression container;
	//navexpression that will be replace with new expression
	protected NavExpContainer navexp;
	//new expression that will be insert
	protected VOCLExpression expression;
	//name of new expression(LET,IF,BOOL,CONSTRAINT,NAVEXP)
	protected String name;
	
	private ExpressionType type;
	
	protected Constraint constraint;
	

	
	protected DublicateExpression dublicate=new DublicateExpression();
	


	/**
	 * Constructor of this class
	 */
	public CreateExpressionCommand(){super();}

	/**
	 * Constructor with given command label
	 * @param label - the label (string) for this command
	 */
	public CreateExpressionCommand(String label){super(label);}
	
	/**
	 * Checks if this command can be executed.
	 * This is true, if a container and an expression is
	 * given in this class. 
	 */
	public boolean canExecute() {
		if(container==null || expression==null) return false;
		return true;
	}

	/**
	 * Execution of this command creates a new 
	 * expression.
	 */
	public void execute() {
		expression.setExptype(type);
		createModel(expression);
		constraint.getVariableDeclaration().removeAll(dublicate.oldvardec);
		constraint.getVariableDeclaration().addAll(dublicate.newvardec);	
	}
	
	/**
	 * Overrides the createModel method from 
	 * CreateExpressionCommand
	 */
	protected void createModel(VOCLExpression exp){
		int position=type.getValue();
		switch(position){
		case 0:
			if( container instanceof Constraint){
				Constraint constraint=(Constraint) container;
				constraint.setVOCLExpression(null);
				constraint.setVOCLExpression(exp);
			}else{
				PredicateContainer pred=(PredicateContainer) container;
				pred.setVOCLExpression(null);
				pred.setVOCLExpression(exp);
			}
			break;
		case 1:
			BoolExp boolexp1=(BoolExp)container;
			boolexp1.setOp1(null);
			boolexp1.setOp1(exp);
			break;	
		case 2:
			BoolExp boolexp2=(BoolExp)container;
			boolexp2.setOp2(null);
			boolexp2.setOp2(exp);
			break;	
		case 3:
			IfExp ifexp=(IfExp)container;
			ifexp.setIfExpression(null);
			ifexp.setIfExpression(exp);
			break;
		case 4:
			IfExp thenexp=(IfExp)container;
			thenexp.setThenExpression(null);
			thenexp.setThenExpression(exp);
			break;
		case 5:
			IfExp elseexp=(IfExp)container;
			elseexp.setElseExpression(null);
			elseexp.setElseExpression(exp);
			break;
		case 6:
			LetExp letexp=(LetExp)container;
			letexp.setLetExpression(null);
			letexp.setLetExpression(exp);
			break;
		case 7:	
			LetExp inexp=(LetExp)container;
			inexp.setInExpression(null);
			inexp.setInExpression(exp);
			break;
		}
	}
	
	
	/**
	 * redos an undo
	 */	
	public void redo(){
		dublicate.getOldIsIn().deleteAllIsIn();
		createModel(expression);
		constraint.getVariableDeclaration().removeAll(dublicate.oldvardec);
		constraint.getVariableDeclaration().addAll(dublicate.newvardec);
		dublicate.getNewIsIn().createAllIsIn();
	}
	
	/**
	 * invalidates an action done by this command
	 */
	public void undo(){
		//FIXME testen!!!!!
		dublicate.getNewIsIn().deleteAllIsIn();
		createModel(navexp);
		constraint.getVariableDeclaration().addAll(dublicate.oldvardec);
		constraint.getVariableDeclaration().removeAll(dublicate.newvardec);
		dublicate.getOldIsIn().createAllIsIn();
	}
	

	/**
	 * Retrieves the expression of this command
	 * @return - the expression of this command
	 */
	public VOCLExpression getExpression(){return expression;}

	/**
	 * Sets the expression for this command
	 * @param the new expression 
	 */
	public void setExpression(VOCLExpression expression){
		this.expression = expression;
	}

	/**
	 * Retrieves the name of this expression
	 * @return - the name as string
	 */
	public String getName() {return name;}

	/**
	 * Sets the new name of this expression
	 * @param string- the name
	 */
	public void setName(String string) {name = string;}

	/**
	 * Retrieves teh container, where the expression
	 * is stored inside.
	 * @return - the container
	 */
	public VOCLExpression getContainer() {return container;}

	/**
	 * Sets the container where thsi expression should be 
	 * placed inside.
	 * @param expression - the container
	 */
	public void setContainer(VOCLExpression expression) {
		container = expression;
	}

	/**
	 * @return
	 */
	public NavExpContainer getNavexp() {return navexp;}

	/**
	 * @param container
	 */
	public void setNavexp(NavExpContainer container) {
		type=container.getExptype();
		navexp = container;
		constraint=null;
		VOCLExpression exp=(VOCLExpression)navexp;
		while(constraint==null){
			if(exp.eContainer() instanceof Constraint)
				constraint=(Constraint) exp.getConstraint();
			else exp=(VOCLExpression) exp.eContainer();	
		}
	}

}
