/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
 
package vocleditor.commands.delete;

import VOCLModel.ClassifierNode;
import VOCLModel.Constraint;
import VOCLModel.NavExpContainer;
import VOCLModel.PredicateContainer;
import VOCLModel.SetClassifierRole;
import VOCLModel.VariableDeclaration;

/**
 * @author eddi
 */
public class DeleteSetClassifierRole extends DeleteClassifierNode{


	private  PredicateContainer predicate=null;
	private SetClassifierRole classifier;

	/**
	 * @param constraint
	 * @param var
	 * @param parent
	 * @param delClassifier
	 */
	public DeleteSetClassifierRole(Constraint constraint, VariableDeclaration var,
									NavExpContainer parent, ClassifierNode delClassifier) {
		super(constraint, var, parent, delClassifier);
		classifier=(SetClassifierRole) delClassifier;
	}
	
	
	/* (non-Javadoc)
	 * @see vocleditor.commands.delete.DeleteClassifierNode#undo()
	 */
	public void undo() {
		super.undo();
		if(predicate!=null)
			parent.setPredicateContainer(predicate);
	}
	/**
	 * Execution of this command deletes a classifierRole
	 * completely.
	 */
	public void delete() {
		predicate=classifier.getPredicateContainer();
		if(predicate!=null){
			DeleteVOCLExpression deleteExp=new DeleteVOCLExpression(variablen);
			deleteExp.deleteExpression(predicate.getVOCLExpression());
			variablen.add(predicate.getIterator());
			constraint.getVariableDeclaration().remove(predicate.getIterator());
			((NavExpContainer)predicate.eContainer()).setPredicateContainer(null);
			
		}
			if(classifier.getOpVar()!=null){
				variablen.add(classifier.getOpVar());
				constraint.getVariableDeclaration().remove(classifier.getOpVar());
			}	
			if(classifier.getIteratorVar()!=null){
				variablen.add(classifier.getIteratorVar());
				constraint.getVariableDeclaration().remove(classifier.getIteratorVar());
			}	

	}


	/**
	 * @return Returns the predicate.
	 */
	public PredicateContainer getPredicate() {
		return predicate;
	}
	/* (non-Javadoc)
	 * @see vocleditor.commands.delete.DeleteClassifierNode#addVariable()
	 */
	public void addVariable() {
		if(var!=null)
				constraint.getVariableDeclaration().add(var);

	}
	/* (non-Javadoc)
	 * @see vocleditor.commands.delete.DeleteClassifierNode#deleteVariable()
	 */
	public void deleteVariable() {
		if(var!=null){
			if(constraint.getVariableDeclaration().contains(var))
				constraint.getVariableDeclaration().remove(var);
		}	
	}
	/* (non-Javadoc)
	 * @see vocleditor.commands.delete.DeleteClassifierNode#deleteNode()
	 */
	public void deleteNode() {
		if(parent.getClassifierNode().contains(classifier))
			parent.getClassifierNode().remove(classifier);	
	}
	/* (non-Javadoc)
	 * @see vocleditor.commands.delete.DeleteClassifierNode#addNode()
	 */
	public void addNode() {
		parent.getClassifierNode().add(classifier);	
	}
	
}
