/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.commands.delete;

import java.util.Iterator;
import java.util.List;

import vocleditor.model.IsIn;

import VOCLModel.Attribute;
import VOCLModel.BoolExp;
import VOCLModel.ClassifierNode;
import VOCLModel.ClassifierRole;
import VOCLModel.IfExp;
import VOCLModel.LetExp;
import VOCLModel.Method;
import VOCLModel.NavExpContainer;
import VOCLModel.PredicateContainer;
import VOCLModel.SetClassifierRole;
import VOCLModel.VOCLExpression;


/**
 * @author eddi
 */
public class DeleteVOCLExpression{

	private List vardec;
	private IsIn listIsIn=new IsIn();

	
	/**
	 * 
	 */
	public DeleteVOCLExpression(List vardec) {
		this.vardec=vardec;
	}
	
	//delete NavExpContainer from the Model
	private void deleteNavExpContainer(NavExpContainer exp){
		if(exp.getPredicateContainer()!=null){
			deleteExpression(exp.getPredicateContainer().getVOCLExpression());	
			vardec.add(exp.getPredicateContainer().getIterator());
		}
		List listnode=exp.getClassifierNode();
		Iterator nodeiterator=listnode.iterator();
		//for each ClassifierNode put callVar in the list
		while(nodeiterator.hasNext()){
			ClassifierNode node=(ClassifierNode) nodeiterator.next();
			if(node.getCallVar()!=null){
				//insert CallVar in the list
				if(node instanceof ClassifierRole){
				    PredicateContainer predicate=((ClassifierRole)node).getCallVar().getPredicateContainer();
					if(predicate==null){
						vardec.add(node.getCallVar());
					}else{
					    listIsIn.addAndDeleteIsIn(predicate.getPredicat(),
					            				  (ClassifierRole)node,
					            				  (NavExpContainer) predicate.getPredicat().eContainer());
					}
				}else{
					vardec.add(node.getCallVar());
				}
			}
			
			//for each attribute of ClassifierNode put VariableDeclaration in the list	
			List listattribute=node.getAttribute();	
			Iterator attributeiterator=listattribute.iterator();
			while(attributeiterator.hasNext()){
				Attribute att=(Attribute) attributeiterator.next();
				if(att.getVariableDeclaration()!=null){
					//insert VariableDeclaretion of Attribute in the list
					vardec.add(att.getVariableDeclaration());
				}
			}
		
			//for each CollClassifierRole put iteratorVar and opVar in the list
			if(node instanceof SetClassifierRole){
				SetClassifierRole collcl=(SetClassifierRole) node;
				if(collcl.getIteratorVar()!=null){
					//insert IteratorVar in the list
					vardec.add(collcl.getIteratorVar());
				}
				if(collcl.getOpVar()!=null){
					//insert OpVar in the list
					vardec.add(collcl.getOpVar());	
				}
			}
			
			//for each method of CollClassifierRole put call and rReturn in the list
			if(node instanceof ClassifierRole){
				ClassifierRole simplecl=(ClassifierRole) node;
				if(simplecl.getMethod()!=null){
					Method method=simplecl.getMethod();
					if(!method.getCall().isEmpty()){
						//insert all parametors of method in the list 
						vardec.addAll(method.getCall());
					}
					if(method.getRReturn()!=null){
						//insert return of method in the list
						vardec.add(method.getRReturn());
					}
				}
			}
		}
	}
	
	//delete BoolExp from the Model
	private void deleteBoolExp(BoolExp exp){
		if(exp.getOp1()!=null)
			deleteExpression(exp.getOp1());
		if(exp.getOp2()!=null)
			deleteExpression(exp.getOp2());
		
	}
	
	//delete LetExp from the Model
	private void deleteLetExp(LetExp exp){
		if(exp.getInExpression()!=null)
			deleteExpression(exp.getInExpression());
		if(exp.getLetExpression()!=null)
			deleteExpression(exp.getLetExpression());
	}
	
	//delete IfExp from the Model
	private  void deleteIfExp(IfExp exp){
		if(exp.getIfExpression()!=null)
			deleteExpression(exp.getIfExpression());
		if(exp.getThenExpression()!=null)
			deleteExpression(exp.getThenExpression());
		if(exp.getElseExpression()!=null)
			deleteExpression(exp.getElseExpression());
	}
	
	//delete VOCLExpression from the Model
	public  void deleteExpression(VOCLExpression exp){
		if(exp instanceof BoolExp){
			deleteBoolExp((BoolExp) exp);
		}else if(exp instanceof NavExpContainer){
			deleteNavExpContainer((NavExpContainer)exp);
		}else if(exp instanceof LetExp){
			deleteLetExp((LetExp) exp);
		}else if(exp instanceof IfExp){
			deleteIfExp((IfExp) exp);
		}
	}


    /**
     * @return Returns the listIsIn.
     */
    public IsIn getListIsIn() {
        return listIsIn;
    }
}
