/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
 
package vocleditor.commands.property;


import org.eclipse.gef.commands.Command;
import vocleditor.figure.NavExpContainerFigure;
import VOCLModel.VOCLExpression;


/**
 * @author vilapower
 *
 * This Command is created by the Editpart: either a navigation expression
 * or a boolean expression. It changes the boolean attribute negated of the model. 
 */
public class VOCLExpNegateCommand extends Command {
	
	// Names for Label and Description of this Command
	private static final String ConnectionCommand_Label 		= "negate";
	private static final String ConnectionCommand_Description 	= "Negates a VOCL-Expression";
	
	// the associated VOCL-Expression Container
	protected VOCLExpression container;
	protected NavExpContainerFigure figure;


	/**
	 * Constructor, simply calls the superclass
	 * with the ConnectionLabel as parameter.
	 */
	public VOCLExpNegateCommand() {
		super(ConnectionCommand_Label);
	}

	/**
	 * This method returns true, if a source (VOCLExpressionContainer, 
	 * see class-members) is specified. 
	 */
	public boolean canExecute() {
		if (container == null)
			return false;
		else
			return true;
	}
	
	/**
	 * This method handles the execution of negating an VOCL-Expression
	 * Finally the VOCLExpressionFigure is updated.
	 */
	public void execute() {
		// The action 	
			this.executeUpdate();

	}

	/**
	 * Retrieves the description.
	 */
	public String getLabel() {
		return ConnectionCommand_Label; // Description; // test Label instead
	}

	/**
	 * This method provides the undo functionality, by simply
	 * calling executeUpdate to swap the negation Attribute of the model.
	 * 
	 */
	public void undo() {
		this.executeUpdate();
	}


	/**
	 * Redo this action means 
	 * simply call executeUpdate .
	 */
	public void redo() {
		this.executeUpdate();
	}

	/**
	 * Changes the value of the boolean Attribute negated
	 * in the model.
	 */
	private void executeUpdate() {
		boolean negated = container.isIsNegated();
		container.setIsNegated(!(negated));
	}

	/**
	 * Returns the container-class of this model.
	 * @return - a VOCLExpressionContainer
	 */
	public VOCLExpression getContainer() {
		return container;
	}

	/**
	 * Sets the container-class for this model.
	 * @param container
	 */
	public void setContainer(VOCLExpression container) {
		this.container = container;
	}

}