/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.edit;


import java.util.LinkedList;
import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editparts.AbstractConnectionEditPart;

import vocleditor.edit.policy.AssociationRoleEditPolicy;
import vocleditor.edit.policy.AssociationRoleEndpointEditPolicy;
import vocleditor.edit.policy.AssociationRoleSelectionHandlesEditPolicy;
import vocleditor.figure.AssociationRoleFigure;
import VOCLModel.AssociationRole;

/**
 * Class       :AssociationRoleEditPart<br/>
 * Package     : vocleditor.edit<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : Class to provide application for AssociationRole
 * <br/>
 * @author vilapower
 */
public class AssociationRoleEditPart extends AbstractConnectionEditPart {


	/**
	 * Creates a new AssociationRoleEditPart instance.
	 * @param element AssociationRole
	 */
	public AssociationRoleEditPart(AssociationRole asr) {
		super();
		setModel(asr);
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		installEditPolicy( EditPolicy.CONNECTION_ENDPOINTS_ROLE, new AssociationRoleEndpointEditPolicy() );
		installEditPolicy( EditPolicy.CONNECTION_BENDPOINTS_ROLE, new AssociationRoleSelectionHandlesEditPolicy() );
		installEditPolicy( EditPolicy.CONNECTION_ROLE, new AssociationRoleEditPolicy());
	}

	/** 
	 * Return the Model of this EditPart as AssociationRole.
	 * @see VOCLModel4.AssociationRole
	 *@return  AssociationRole .
	 */
	public AssociationRole getAssociationRole(){
		  return (AssociationRole) getModel();
	  }



	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure(){
		AssociationRoleFigure f=new AssociationRoleFigure();		
		return f;
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	protected List getModelChildren() {
		// Auto-generated method stub
		List children = new LinkedList();
		children.add(getAssociationRole().getSource());
		children.add(getAssociationRole().getTarget());	
		return children;
	}



}
