/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.edit.policy;


import org.eclipse.draw2d.BorderLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PrecisionRectangle;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import vocleditor.commands.constraints.MoveSashCommand;

import vocleditor.edit.VOCLSashEditPart;
import vocleditor.figure.VOCLBorderLayout;
import VOCLModel.IfExp;
import VOCLModel.VOCLSash;






public class IfExpXYLayoutEditPolicy extends VOCLExpressionXYLayoutEditPolicy {
	
	private double proportionH;
	private double proportionV;


	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createChangeConstraintCommand(org.eclipse.gef.EditPart, java.lang.Object)
	 */
	protected Command createChangeConstraintCommand(EditPart child, Object constraint) {
		if(child instanceof VOCLSashEditPart){
			VOCLSash sash=(VOCLSash) child.getModel();
			if(sash.isVertikal())
				return new MoveSashCommand((IfExp)getHost().getModel(),proportionV,true);
			else
				return new MoveSashCommand((IfExp)getHost().getModel(),proportionH,false);
		}
		return null;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#getConstraintFor(org.eclipse.gef.requests.ChangeBoundsRequest, org.eclipse.gef.GraphicalEditPart)
	 */
	protected Object getConstraintFor(ChangeBoundsRequest request,GraphicalEditPart child) {
		if(child instanceof VOCLSashEditPart){
			IFigure container = child.getFigure().getParent();
			Rectangle p=container.getClientArea();
			Point st=container.getClientArea().getLocation();
			container.translateToAbsolute(st);
			Rectangle rect = new PrecisionRectangle(child.getFigure().getBounds());
			child.getFigure().translateToAbsolute(rect);
			Rectangle original = rect.getCopy();
			rect = request.getTransformedRectangle(rect);	
			VOCLSash sash=(VOCLSash)((VOCLSashEditPart) child).getModel();
			
			if(sash.isVertikal())proportionV=((double)(rect.x - st.x))/(p.width);
			else proportionH=((double)(rect.y - st.y))/(p.height);
			
			if(proportionH<0.01)proportionH=0.05;		
			if(proportionH>1)proportionH=0.95;	
			if(proportionV<0.01)proportionV=0.05;		
			if(proportionV>1)proportionV=0.95;	
			
			return VOCLBorderLayout.CENTER;
		}	
		return BorderLayout.CENTER;
		
	}


	

}
