/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.figure;


import org.eclipse.draw2d.ToolbarLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.emf.common.util.EList;
import org.eclipse.swt.graphics.Image;

import VOCLModel.Attribute;
import VOCLModel.SetClassifierRole;
import VOCLModel.VariableDeclaration;



public class SetClassifierRoleFigure extends AbstractNodeFigure {

	private int setType = 0;
	private String setVar = "";
	
	private Image emptyImage;
	private Image notEmptyImage;
	private ShadowClassFigure classFigure;
	private SetOperationFigure setFigure;

	/**
	 * Constructor of the ClassifierRoleFigure:
	 * Creates a Figure for a class with a class-name,
	 * a variable-name (optional - may be empty, but not null),
	 * and attributes and place them inside a UML-like
	 * box for classes.
	 * @param varname   - the variable-name for this class-instance
	 * @param classname - the class name
	 */
	public SetClassifierRoleFigure(String varname, String classname, SetClassifierRole ccr) {
		super(varname,classname);
		// and a LineBorder
		ToolbarLayout layout = new ToolbarLayout(true);
		setLayoutManager(layout);
		setOpaque(true);		
		
		// create and add the classFigure
		classFigure = new ShadowClassFigure(varname, classname);
		EList attributes = ccr.getAttribute();
		for (int i = 0; i < attributes.size(); i++) {
			Attribute att = (Attribute)attributes.get(i);
			if (att.getVariableDeclaration() != null) {
				classFigure.addAttribute(att.getName(), att.getType(), 
					att.getVariableDeclaration().getVarName(), false);
			}
			else {
				classFigure.addAttribute(att.getName(), att.getType(), 
					att.getValue(), true);
			}
		}
		add(classFigure);
		
		setFigure = new SetOperationFigure();
		
		int type = ccr.getOpKind().getValue();
		String vn = "";
		String summand = "";
		try {
			VariableDeclaration decl = ccr.getOpVar();
			if (decl != null) vn = decl.getVarName();
			
			decl = ccr.getIteratorVar();
			if (decl != null) summand = decl.getVarName();
		} catch (Exception e) {
			vn = "";
			summand = "";
		}
		
		try {
			setFigure.setOperationType(type, vn, summand);
		} catch (Exception e) {}	
		add(setFigure);

		createVoclEditorAnchors();
	}
	
	
	/**
	 * A method for specifying the variable- and classname
	 * of this ClassifierRoleFigure.
	 * Changes only the label, but not the values.
	 * @param varname 	- the name of the variable
	 * @param classname	- the name of the class
	 */
	public void setName(String varname,String classname){
		classFigure.setClassifierText(varname,classname);	
	}
		
	/**
	 * Retrieving the minimum Dimension
	 * @see getPreferredSize()
	 */
	public Dimension getMinimumSize(int wHint, int hHint) {		
		return getPreferredSize();
	}

	/**
	 * Retrieving the wanted Dimension for this figure.
	 */
	public Dimension getPreferredSize(int wHint, int hHint) {
		Dimension dim1 = classFigure.getPreferredSize(wHint,hHint).getCopy();
		Dimension dim2 = new Dimension(0,0);
		try {
			dim2 = setFigure.getPreferredSize(wHint,hHint).getCopy();
		} catch (Exception e) {}
		return new Dimension(dim1.width + dim2.width, Math.max(dim1.height,dim2.height));
	}
	
	
	/**
	 * This method will set the ClassVarName of this 
	 * classifier in the graphical view.
	 */
	public void setClassVarName(String name) {
		classFigure.setVarName(name);
	}
	
	/**
	 * Delegate the SetOperationFigure
	 * @param type - the type
	 * @param var - the varName for setSize
	 */
	public void setSetOperation(int type, String var, String summand) {
		setFigure.setOperationType(type, var, summand);
	}

	/**
	 * Delegates the addAttribute Command to the
	 * classFigure (which is a ShadowClassFigure,
	 * which encapsulate a SimpleClassifierRoleFigure)
	 * @param name - an attributes name
	 * @param type - an attributes type
	 * @param value - an attributes value
	 */
	public void addAttribute(String name, String type, String value, boolean isValue) {
		classFigure.addAttribute(name, type, value, isValue);
		
	}
	
	public void removeAttribute(int i) {
		classFigure.removeAttribute(i);
	}

	public void removeAllAttributes() {
		classFigure.removeAllAttributes();
	}
}
