/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.figure;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.FlowLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.swt.graphics.Image;


import vocleditor.editors.VOCLEditorConstants;

import VOCLModel.OpKind;

public class SetOperationFigure extends Figure {
	
	// images for displaying the type
	// and label where all types are inside
	private Label display;
	private Image emptyImage;
	private Image notEmptyImage;
	private Image sumImage;
	
	String varName;
	String summandName;
	int type = 0;

	/**
	 * Class-Constructor for the SetOperationFigure
	 * which shall display an image (empty/not empty set) 
	 * or a label for set-size;
	 */
	public SetOperationFigure() {
		initializeImages();
		
		// building the Figure
		FlowLayout layout = new FlowLayout(false);
		layout.setMinorAlignment(FlowLayout.ALIGN_RIGHTBOTTOM);
		setLayoutManager(layout);
		
		display = new Label();
		display.setBackgroundColor(VOCLEditorConstants.classColor);
		display.setOpaque(true);
		display.setBorder(new LineBorder(ColorConstants.black,1));
		add(display);
		this.setPreferredSize(new Dimension(0,0));
		
	}

	/**
	 * Retrieving the wanted Dimension for this figure.
	 * It is (0,0) if no operation on this set is
	 * specified.
	 */
	public Dimension getPreferredSize(int wHint, int hHint) {
		switch(this.type) {
			case OpKind.NO_OP: return new Dimension(0,0);
			default: return display.getPreferredSize();
		}
	}
	
	/**
	 * This method is called when a new
	 * Operation type for the collection
	 * is specified
	 * @param type - the type of the operation
	 * @param varName - the variables name (if needed)
	 */
	public void setOperationType(int type, String varName, String summand) {
		this.varName = varName;
		this.summandName = summand;
		switch(type) {
			case OpKind.NO_OP:
			case OpKind.EMPTY:
			case OpKind.NOT_EMPTY:
			case OpKind.COUNT:
			case OpKind.SUM:
				this.type = type;
				break;
		}
		this.generateNewDisplay();	
	}
	
	/**
	 * This method generates the new displayd
	 * Label depending on the type
	 */
	private void generateNewDisplay() {
		switch(this.type) {
			case OpKind.NO_OP:
				display.setText("");
				display.setIcon(null);
				break;
			case OpKind.EMPTY:
				display.setText("");
				display.setIcon(emptyImage);
				break;
			case OpKind.NOT_EMPTY:
				display.setText("");
				display.setIcon(notEmptyImage);
				break;
			case OpKind.COUNT:
				display.setText("#" + varName);
				display.setIcon(null);
				break;
			case OpKind.SUM:
				display.setText(summandName + " = " + varName);
				display.setIcon(sumImage);
		}
	}

	/**
	 * Method for loading images
	 */
	public void initializeImages() {
		// initalizing the images
	   emptyImage = new Image(null, 
		SetClassifierRoleFigure.class.getResourceAsStream("gif/empty.gif"));
		
	   notEmptyImage = new Image(null, 
		SetClassifierRoleFigure.class.getResourceAsStream("gif/not_empty.gif"));
	
	   sumImage = new Image(null, 
		SetClassifierRoleFigure.class.getResourceAsStream("gif/sum.gif"));
	}

}
